//
// Hi-Tech Lite source file for DB038 board with 16F887:
// A/D converter functions
//
// $Id: db038-adc.c,v 1.3 2012/04/24 17:25:14 Wouter Exp $
//
// (c) Wouter van Ooijen - wouter@voti.nl
//
// Copying and distribution of this file, with or without modification,
// are permitted in any medium without royalty provided the copyright
// notice and this notice are preserved.  This file is offered as-is,
// without any warranty.
// 

#include "db038-adc.h"

#if defined(_16F887)

unsigned int adc_channel_read( unsigned char channel ){ 
	
	 // Fosc/32; channel; ADON 
   ADCON0 = 0x41 + ( channel << 2 ); 
   
   // right justified; Vss/Vdd ref
   ADCON1 = 0x80; 
   
   // configure the pin as analog
   if( channel < 8 ){ 
      ANSEL |= ( 1 << channel );
   } else if( channel < 14 ){
      ANSELH |= ( 1 << ( channel - 8 ));	
   }
   
   // configure the pin as input
   switch( channel ){
      case  0 : TRISA0 = 1; break;
      case  1 : TRISA1 = 1; break;
      case  2 : TRISA2 = 1; break;
      case  3 : TRISA3 = 1; break;
      case  4 : TRISA5 = 1; break;
      case  5 : TRISE0 = 1; break;
      case  6 : TRISE1 = 1; break;
      case  7 : TRISE2 = 1; break;
      case  8 : TRISB2 = 1; break;
      case  9 : TRISB3 = 1; break;
      case 10 : TRISB1 = 1; break;
      case 11 : TRISB4 = 1; break;
      case 12 : TRISB0 = 1; break;
      case 13 : TRISB5 = 1; break;
      default: break;
   }
   
   // p107 requires 4.67 us
   WAIT_US( 5 );
   
   // start conversion and wait for completion
   ADCON0 |= 2;
   while( ADCON0 & 2 ){}
   
   // return A/D conversion result
   return ( ((unsigned int) ADRESH ) << 8 ) + ADRESL;	
}

#elif defined(__18F4520)

unsigned int adc_channel_read( unsigned char channel ){ 

   // set channel, ADON
   ADCON0 = ( channel << 2 ) + 0x01;

   // configure the pin as input
   switch( channel ){
      case  0 : TRISAbits.TRISA0 = 1; break;
      case  1 : TRISAbits.TRISA1 = 1; break;
      case  2 : TRISAbits.TRISA2 = 1; break;
      case  3 : TRISAbits.TRISA3 = 1; break;
      case  4 : TRISAbits.TRISA5 = 1; break;
      case  5 : TRISEbits.TRISE0 = 1; break;
      case  6 : TRISEbits.TRISE1 = 1; break;
      case  7 : TRISEbits.TRISE2 = 1; break;
      case  8 : TRISBbits.TRISB2 = 1; break;
      case  9 : TRISBbits.TRISB3 = 1; break;
      case 10 : TRISBbits.TRISB1 = 1; break;
      case 11 : TRISBbits.TRISB4 = 1; break;
      case 12 : TRISBbits.TRISB0 = 1; break;
      case 13 : TRISBbits.TRISB5 = 1; break;
      default: break;
   }
   
   // Vss/Vdd ref, select analog input
   ADCON1 = 0x0E - channel;

	 // Fosc/32; right justified
   ADCON2 = 0x82;
   
   // p228 requires 2.4 us
   WAIT_US( 5 );
   
   // start conversion and wait for completion
   ADCON0 |= 2;
   while( ADCON0 & 2 ){}

   // return A/D conversion result
   return ( ((unsigned int) ADRESH ) << 8 ) + ADRESL;	
}

#else
   #error unsupported chip
#endif

unsigned int adc_microphone( void ){
   TRISE = 0;
   PORT_SET( PORTE, 2 );
   return adc_channel_read( 0 );	
}

unsigned int adc_battery( void ){
   TRISE = 0;
   PORT_SET( PORTE, 3 );
   return adc_channel_read( 0 );	
}

unsigned int adc_potmeter( void ){
   TRISE = 0;
   PORT_SET( PORTE, 4 );
   return adc_channel_read( 0 );	
}

unsigned int adc_ldr( void ){
   TRISE = 0;
   PORT_SET( PORTE, 5 );
   return adc_channel_read( 0 );	
}

unsigned int adc_lm335( void ){
   TRISE = 0;
   PORT_SET( PORTE, 6 );
   return adc_channel_read( 0 );	
}

unsigned int adc_tl431( void ){
   TRISE = 0;
   PORT_SET( PORTE, 7 );
   return adc_channel_read( 0 );	
}

unsigned int adc_current( void ){
   TRISE = 0;
   PORT_SET( PORTE, 3 );
   return adc_channel_read( 1 );	
}
