//
// Hi-Tech Lite source file for DB038 board with 16F887:
// Dallas 1-wire functions
//
// $Id: db038-dallas.c,v 1.4 2012/04/24 17:25:16 Wouter Exp $
//
// (c) Wouter van Ooijen - wouter@voti.nl
//
// Copying and distribution of this file, with or without modification,
// are permitted in any medium without royalty provided the copyright
// notice and this notice are preserved.  This file is offered as-is,
// without any warranty.
//

#include "db038-dallas.h"

unsigned char d1w_present( void ){
   unsigned char x1, x2;
   D1W_INIT;
   D1W_LOW;
   WAIT_US( 600 );
   D1W_HIGH;
   WAIT_US( 72 );
   x1 = D1W_LINE;
   WAIT_US( 600 );
   x2 = D1W_LINE;
   return ( ! x1 ) & x2;
}

void d1w_byte_send( unsigned char x ){
   unsigned char i;
   D1W_INIT;
   for( i = 0; i < 8; i++ ){
      if( x & 1 ){
         D1W_LOW;
         WAIT_US(   5 );
         D1W_HIGH;
         WAIT_US(  85 );
      } else {
         D1W_LOW;
         WAIT_US(  80 );
         D1W_HIGH;
         WAIT_US(  20 );
      }
      x = x >> 1;
   }
}

unsigned char d1w_byte_receive( void ){
   unsigned char i, d = 0;
   D1W_INIT;
   for( i = 0; i < 8; i++ ){
      d = d >> 1;
      D1W_LOW;
      WAIT_US(  5 );
      D1W_HIGH;
      WAIT_US(  5 );
      if( D1W_LINE ){
         d = d | 0x80;
      }
      WAIT_US( 100 );
   }
   return d;
}

unsigned char d1w_family_code_read( void ){
   unsigned char d1, d2;
   (void)d1w_present();
   d1w_byte_send( 0x33 );
   return d1w_byte_receive();
}

void ds18s20_conversion_start( void ){
   (void)d1w_present();
   d1w_byte_send( 0xCC );
   d1w_byte_send( 0x44 );
}

unsigned int ds18s20_temperature_read( void ){
   unsigned char d1, d2;
   (void)d1w_present();
   d1w_byte_send( 0xCC );
   d1w_byte_send( 0xBE );
   d1 = d1w_byte_receive();
   d2 = d1w_byte_receive();
   return (((unsigned int)d2) << 8 ) + d1;
}