//
// Hi-Tech Lite source file for DB038 board with 16F887:
// I2C functions
//
// $Id: db038-i2c.c,v 1.4 2012/04/24 17:25:19 Wouter Exp $
//
// (c) Wouter van Ooijen - wouter@voti.nl
//
// Copying and distribution of this file, with or without modification,
// are permitted in any medium without royalty provided the copyright
// notice and this notice are preserved.  This file is offered as-is,
// without any warranty.
// 

#include "db038-i2c.h"

void i2c_init( void ){
   disable_analog_functions();
   TRISA |= 0x03; 	
   TRISE &= 0xF8; 	
   PORT_SET( PORTE, 1 );
   PIN_CLEAR( PORTA, 0 );
   PIN_CLEAR( PORTA, 1 );
   i2c_stop();
}

void i2c_start( void ){
   I2C_SDA_SET( 1 );
   I2C_SCL_SET( 1 );
   I2C_SDA_SET( 0 );
   I2C_SCL_SET( 0 );
}

void i2c_stop( void ) {
   I2C_SDA_SET( 0 );
   I2C_SCL_SET( 0 );
   I2C_SCL_SET( 1 );
   I2C_SDA_SET( 1 );
}

void i2c_ack( void ){
   I2C_SDA_SET( 0 );
   I2C_SCL_SET( 1 );
   I2C_SCL_SET( 0 );
}

void i2c_nack( void ){
   I2C_SDA_SET( 1 );
   I2C_SCL_SET( 1 );
   I2C_SCL_SET( 0 );
}

void i2c_wait_ack( void ){
   I2C_SDA_SET( 1 );
   I2C_SCL_SET( 1 );
   // wait for clock low
   // check that data is low
   I2C_SCL_SET( 0 );
}

void i2c_byte_write( unsigned char x ){
	 unsigned char i;
	 for( i = 0; i < 8; i++ ){
	 	  I2C_SDA_SET( x & 0x80 );
	 	  I2C_SCL_SET( 1 );
	 	  I2C_SCL_SET( 0 );
	    x = x << 1;	
	 }
}

unsigned char i2c_byte_read( void ){
   unsigned char x;
   unsigned char i;
   I2C_SDA_SET( 1 );
   for( i = 0; i < 8; i++ ){
      I2C_SCL_SET( 1 );
	  x = x << 1;	
	  if( I2C_SDA_READ ){
         x |= 0x01;	 	  	
	  }
	  I2C_SCL_SET( 0 );
   }
   return x;
}

void i2c_read( 
   unsigned char address,
   unsigned char data[],
   unsigned char n
){
   unsigned char i;
   i2c_init();
   i2c_start();
   i2c_byte_write( ( address << 1 ) + 0x01 );
   i2c_wait_ack();
   for( i = 0; i < n; i++ ){
      data[ i ] = i2c_byte_read();
      i2c_ack();	
   }
   i2c_stop();
}

void i2c_write( 
   unsigned char address,
   const unsigned char data[],
   unsigned char n
){
   unsigned char i;
   i2c_init();
   i2c_start();
   i2c_byte_write( ( address << 1 ) + 0x00 );
   i2c_wait_ack();
   for( i = 0; i < n; i++ ){
      i2c_byte_write( data[ i ] );
      i2c_ack();	
   }
   i2c_stop();
}
