//
// Hi-Tech Lite include file for DB038 board with 16F887:
// I2C functions
//
// $Id: db038-i2c.h,v 1.4 2012/04/24 17:25:20 Wouter Exp $
//
// (c) Wouter van Ooijen - wouter@voti.nl
//
// Copying and distribution of this file, with or without modification,
// are permitted in any medium without royalty provided the copyright
// notice and this notice are preserved.  This file is offered as-is,
// without any warranty.
// 

#ifndef _DB038_I2C_
#define _DB038_I2C_

#include "db038.h"

// RA0 == SDA
// RA1 == SCL

#define i2c_delay do { \
   WAIT_US( 25 ); \
} while( 0 )   

#define I2C_SDA_SET( x ) do { \
   if( x ){ \
      PIN_DIRECTION( PORTA, 0, INPUT ); \
   } else { \
   	  PIN_DIRECTION( PORTA, 0, OUTPUT ); \
   } \
   i2c_delay; \
} while( 0 ) 

#define I2C_SCL_SET( x ) do { \
   if( x ){ \
      PIN_DIRECTION( PORTA, 1, INPUT ); \
   } else { \
   	  PIN_DIRECTION( PORTA, 1, OUTPUT ); \
   } \
   i2c_delay; \
} while( 0 ) 

#define I2C_SDA_READ ( PORTA & 0x01 )
#define I2C_SCL_READ ( PORTA & 0x02 )

void i2c_init( void );
void i2c_start( void );
void i2c_stop( void );
void i2c_ack( void );
void i2c_nack( void );
void i2c_byte_write( unsigned char x );
unsigned char i2c_byte_read( void );
void i2c_read( 
   unsigned char address,
   unsigned char data[],
   unsigned char n
);
void i2c_write(
   unsigned char address,
   const unsigned char data[],
   unsigned char n
);

#endif

#define I2C_TCN75_ADDRESS 0x48
#define I2C_24C01_ADDRESS 0x50

