//
// Hi-Tech Lite source file for DB038 board with 16F887:
// UART functions
//
// $Id: db038-uart.c,v 1.5 2010/06/19 07:16:56 Wouter Exp $
//
// (c) Wouter van Ooijen - wouter@voti.nl
//
// Copying and distribution of this file, with or without modification,
// are permitted in any medium without royalty provided the copyright
// notice and this notice are preserved.  This file is offered as-is,
// without any warranty.
//
// 2010-10-19 : "((unsigned int)spbrg)" correct for compiler bug
// 

#include "db038-uart.h"
#include "db038-string.h"

// BRGH=1, BRG16=1
#define UART_INIT( spbrg )    \
   TRISC6 = 0;		          \
   TRISC7 = 1;                \
   BAUDCTL = 0x08;            \
   SPBRGH = ((unsigned int)spbrg) >> 8;     \
   SPBRG = ((unsigned int)spbrg) & 0xFF;    \
   RCSTA = 0x90;              \
   TXSTA = 0x24;              \

void uart_init_1200( void ){ UART_INIT( 4166 ); }
void uart_init_2400( void ){ UART_INIT( 2082 ); }
void uart_init_4800( void ){ UART_INIT( 1041 ); }
void uart_init_9600( void ){ UART_INIT( 520 ); }
void uart_init_19200( void ){ UART_INIT( 259 ); }
void uart_init_57600( void ){ UART_INIT( 86 ); }
void uart_init_115200( void ){ UART_INIT( 42 ); }
void uart_init_250000( void ){ UART_INIT( 20 ); }

bit uart_char_available( void ){
   return RCIF;
}

char uart_char_receive( void ){
	 while(!RCIF) continue;
   return RCREG;
}

bit uart_send_idle( void ){
   return TXIF;
}

void uart_char_send( char c ){
   while( !TXIF ) continue;
	 TXREG = c;
}

void uart_string_send( const char *p ){
    while( *p != 0 ){
       uart_char_send( *p );
       p++;
    }
}

void uart_string_receive( 
   char *received,
   unsigned char received_size, 
   unsigned int milliseconds, 
   const char * terminate,
   const char * ignore 
){
   unsigned char i, t;
   char c;   
   t = 0;	 
   i = 0;
   for(;;){
   	  received[ i ] = '\0';
      if( i + 1 == received_size ){
         return;
      }
      if( uart_char_available() ){
      	 c = uart_char_receive();
         if( ! char_in( c, ignore )){
            received[ i ] = c;
            i++;
         }
         if( char_in( c, terminate )){
            return;
         }
      }	else {
   	     WAIT_US( 100 );
         t++;
         if( t == 10 ){
   	        t = 0;
            milliseconds--;
            if( milliseconds == 0 ){
               return;
            }
         }
      }   
   }   	
}	

void dmx_char_send( char c ){
   while( !TXIF ) continue;
   WAIT_US( 12 );
   TXREG = c;
}

void dmx_send( 
   unsigned char base,
   unsigned char data[],
   unsigned char n
){
	 unsigned char i;
   uart_init_250000();
   
   // TTL break level is low
   TRISC6 = 0;
   PIN_CLEAR( PORTC, 6 );
   
   // the break starts a frame
   SPEN = 0;
   WAIT_US( 100 );
  
   // the Mark-After-Break time
   SPEN = 1;
   WAIT_US( 20 ); 
   
   // start code is 0 for dimmers etc 
   dmx_char_send( 0x00 );
   
   // send 0's for unused addresses
   for( i = 0; i < base; i++ ){
//      dmx_char_send( 0x00 );	
      dmx_char_send( 0x80 );		
      base--;
   }
   
   // send the data
   for( i = 0; i < n; i++ ){
//      dmx_char_send( data[ i ] );		
      dmx_char_send( 0x80 );		
   }
   
   // 24 channels is the minimum, so send dummy 0's to fill
   n += base;
   while( n < 24 ){
      dmx_char_send( 0x00 );
      n++;
   }
   
   // MBB = mark after frame
   WAIT_US( 100 );
}
