//
// Hi-Tech Lite test application for DB038 board with 16F887:
// Dallas 1-wire interface
//
// $Id: main-ds18s20.c,v 1.2 2012/04/24 17:25:34 Wouter Exp $
//
// (c) Wouter van Ooijen - wouter@voti.nl
//
// Copying and distribution of this file, with or without modification,
// are permitted in any medium without royalty provided the copyright
// notice and this notice are preserved.  This file is offered as-is,
// without any warranty.
//
// The DS18B20 chips that I tested need a stronger pullup than the 
// default 4k7. An extra 2k2 in parallel did the trick.
// 

#include "db038-leds.h"
#include "db038-lcd.h"
#include "db038-string.h"
#include "db038-dallas.h"

#define SEVEN_ERROR SEGMENT_G
#define SEVEN_MINUS SEGMENT_G

void lcd_print_hex4( unsigned int x ){
   lcd_char_print( char_from_nibble( ( x >> 12 ) & 0x0F ));
   lcd_char_print( char_from_nibble( ( x >>  8 ) & 0x0F ));
   lcd_char_print( char_from_nibble( ( x >>  4 ) & 0x0F ));
   lcd_char_print( char_from_nibble( ( x >>  0 ) & 0x0F ));
}

void show_ds18s20( void ){
   unsigned char minus, i;
   unsigned int x;
   int temp;

   x = ds18s20_temperature_read();         
   ds18s20_conversion_start();

   // show name and raw result on the LCD
   lcd_string_print( "\r18S20 : 0x" );
   lcd_print_hex4( x );
   lcd_string_print( "  " );

   // convert the raw temperature to degrees Celcius
   temp = x;
   minus = 0;
   if( x >= 0x1000 ){
      temp = - temp;
      minus = SEVEN_MINUS;
   }
   temp = temp * 5;

   // for debugging: show value after conversion
   lcd_line( 1 );
   lcd_string_print( "0x" );
   lcd_print_hex4( temp );
   lcd_line( 0 );

   for( i = 0; i < 10; i++ ){
      displays_leds_show(
         minus, 
         seven_from_nibble(( temp % 1000 ) / 100 ),
         seven_from_nibble(( temp %  100 ) / 10 ) + SEGMENT_DP,
         seven_from_nibble(( temp %   10 ) ),
         0,
         200
      );
   }  
}

void show_ds18b20( void ){
   unsigned char minus, i;
   unsigned int x;
   int temp;

   ds18s20_conversion_start();
   wait_ms( 2000 );
   x = ds18s20_temperature_read();         

   // show name and raw result on the LCD
   lcd_string_print( "\r18B20 : 0x" );
   lcd_print_hex4( x );
   lcd_string_print( "  " );

   // convert the raw temperature to degrees Celcius
   temp = x;
   minus = 0;
   if( x >= 0x1000 ){
      temp = - temp;
      minus = SEVEN_MINUS;
   }
   temp = ( temp * 5 ) / 8 ;

   // for debugging: show value after conversion
   lcd_line( 1 );
   lcd_string_print( "0x" );
   lcd_print_hex4( temp );
   lcd_line( 0 );

   for( i = 0; i < 10; i++ ){
      displays_leds_show(
         minus, 
         seven_from_nibble(( temp % 1000 ) / 100 ),
         seven_from_nibble(( temp %  100 ) / 10 ) + SEGMENT_DP,
         seven_from_nibble(( temp %   10 ) ),
         0,
         200
      );
   } 
}

void show( void ){
   unsigned char minus, i;
   unsigned int x;
   int temp;

   lcd_string_print( "\rxxx" );
//   for(;;);

   if( !d1w_present() ){
      lcd_string_print( "\rno 1-wire device" );
      displays_leds_show(
         SEVEN_ERROR, 
         SEVEN_ERROR, 
         SEVEN_ERROR, 
         SEVEN_ERROR, 
         0, 
         50 
      );
      return;
   }

   x = d1w_family_code_read();   
   if( x == 0x10 ){
      show_ds18s20();
      return;
   }
   if( x == 0x28 ){
      show_ds18b20();
      return;
   }

   lcd_string_print( "\rnot a temp sens." );
   displays_leds_show(
      SEVEN_ERROR, 
      SEVEN_ERROR, 
      seven_from_nibble( x >> 4 ),
      seven_from_nibble( x ), 
      0, 
      50 
   );
}

void main(void){
   LCD_INIT_2_16;
   lcd_clear();
   lcd_string_print( "\rxxx" );
   //for(;;);
   for(;;){
      show();
   }
}