//
// Hi-Tech Lite test application for DB038 board with 16F887:
// communicate with a Wavecom mobile modem
//
// $Id: main-wavecom.c,v 1.1 2009/11/19 13:35:36 Wouter Exp $
//
// (c) Wouter van Ooijen - wouter@voti.nl
//
// Copying and distribution of this file, with or without modification,
// are permitted in any medium without royalty provided the copyright
// notice and this notice are preserved.  This file is offered as-is,
// without any warranty.
// 

#include "db038-string.h"
#include "db038-leds.h"
#include "db038-lcd.h"
#include "db038-uart.h"

// send the command, 
// receive the response and save it in the global variable response
char response[ 30 ];
void command( const char *cmd ){
	
	 // clear any previously received characters
   uart_string_receive( response, sizeof( response ), 100, "", "" );
   
   // send the command
   uart_string_send( cmd );
   
   // receive and save the response 
   uart_string_receive( response, sizeof( response ), 100, "", "\x0A\x0D" );
}

// send the command,
// receive the reponse,
// if it does not match the expected response:
//    show command and response on the LCD, and blink the LEDs
void command_and_check( const char *cmd, const char *answer ){
   command( cmd );
   if( ! str_equal( response, answer )){

       if(0){
         // for debugging only; note that thisn will send the texts
         // to the modem, so it will likely disturb the communication!
         uart_string_send( "\n[" );
         uart_string_send( answer );
         uart_string_send( "]\n[" );
         uart_string_send( response );
         uart_string_send( "\]n" );
      }

      // show command and response on the LCD
      lcd_clear();
      lcd_line( 0 ); 
      lcd_string_print( "[" );
      lcd_string_print( cmd );
      lcd_string_print( "]" );
      lcd_line( 1 );
      lcd_string_print( "[" );
      lcd_string_print( response );
      lcd_string_print( "]" );

      // blink the LEDs
      for(;;){
         leds_show( 0xFF, 200 );
         leds_show( 0x00, 200 );      
      }
   }
}      	
            
void main(void){
   unsigned char i;
   uart_init_9600();
   lcd_init( 1, 16 );
   lcd_clear();

   // this ends an SMS text mode (if one was active)
   uart_char_send( 0x1A );

   // no echo please
   command( "ATE0\n" );

   // now check whether we communicate with a wavecom modem
   command_and_check( "AT+CGMI\n", " WAVECOM MODEMOK" );  

   // now checke whether the modem type is as we expect
   command_and_check( "AT+CGMM\n", " MULTIBAND  900E  1800 OK" );  

   // Kitt to show everything is OK
   for(;;){
      for( i = 0; i < 15; i++ ){
         if( i < 8 ){
            leds_show( 1 << i , 100 );  
         } else {
            leds_show( 1 << (16 - i) , 100 );  
         } 
      } 
   }
}